use std::io::Write;
use super::{Error, Sector, SectorWriter};

pub struct BinCueWriter {
    _bin_out:      std::boxed::Box<dyn Write>,
    sector_count: usize
}

impl BinCueWriter {
    pub fn new<T: Write + 'static>(writer: T) -> BinCueWriter {
        BinCueWriter{_bin_out: std::boxed::Box::new(writer), sector_count: 0}
    }
}

impl SectorWriter for BinCueWriter {
    fn write(&mut self, mut sector: Sector) -> Result<(), Error> {
        self.finalize_sector(&mut sector)?;

        match sector {
            Sector::Empty(_)     => Err(Error::not_implemented("Empty sector encoding")),
            Sector::Audio(_)     => Err(Error::not_implemented("Audio sector encoding")),
            Sector::CDData(_)    => Err(Error::not_implemented("CD Data sector encoding")),
            Sector::CDXAData(_)  => Err(Error::not_implemented("CD XA Data sector encoding")),
            Sector::CDXAAudio(_) => Err(Error::not_implemented("CD XA Audio sector encoding"))
        }
    }

    fn sector_written_count(&self) -> usize {
        self.sector_count
    }
}