pub mod bin_cue;

pub use cdtypes::cd::sector::Sector;

use super::{encoder::EncoderFunction, types::CDDesc};
use bin_cue::BinCueWriter;
use std::path::PathBuf;
use tool_helper::{Error, open_output_file};

pub enum ImageType {
    BinCue
}

pub trait SectorWriter {
    fn write(&mut self, sector: Sector) -> Result<(), Error>;
    fn sector_written_count(&self)      -> usize;

    fn finalize_sector(&self, sector: &mut Sector) -> Result<(), Error> {
        let sectors_written = self.sector_written_count();
        if let Some(header) = sector.get_header() {
            let sector_lba = header.get_time().to_lba();
            if sector_lba != sectors_written {
                return Err(Error::from_text(format!("Header LBA {} doesn't match written sector count of {}", sector_lba, sectors_written)));
            }
        }

        sector.finalize();
        Ok(())
    }
}

pub fn write_image(cd_desc: CDDesc, encoder: EncoderFunction, image_type: ImageType, output_path: PathBuf) -> Result<(), Error> {
    match image_type {
        ImageType::BinCue => {
            let mut writer = BinCueWriter::new(open_output_file(output_path)?);

            encoder(cd_desc, &mut writer)
        }
    }
}